@@ -1,6 +1,7 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
+# Copyright 2010 Joanmarie Diggs, Mesar Hameed
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,7 +23,8 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
+__copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
+                "Copyright (c) 2010 Joanmarie Diggs, Mesar Hameed."
 __license__   = "LGPL"
 
 # We're going to force the name of the app to "orca" so pygtk
@@ -80,7 +82,7 @@ import httpserver
 import keynames
 import keybindings
 import orca_state
-import platform
+import orca_platform
 import speech
 
 from input_event import BrailleEvent
@@ -89,6 +91,7 @@ from input_event import MouseButtonEvent
 from input_event import keyEventToString
 
 from orca_i18n import _           # for gettext support
+from orca_i18n import ngettext
 
 if settings.debugMemoryUsage:
     import gc
@@ -115,6 +118,14 @@ _debugSwitch = False
 #
 _debugFile = None
 
+# A subset of the original Xmodmap info prior to our stomping on it.
+# Right now, this is just for the user's chosen Orca modifier(s).
+#
+_originalXmodmap = ""
+_orcaModifiers = settings.DESKTOP_MODIFIER_KEYS + settings.LAPTOP_MODIFIER_KEYS
+_capsLockCleared = False
+_restoreOrcaKeys = False
+
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING PRESENTATION MANAGERS                           #
@@ -348,7 +359,9 @@ def cycleDebugLevel(script=None, inputEvent=None):
 
     debug.debugLevel = levels[levelIndex]
 
-    speech.speak("Debug level %s." % levels[levelIndex + 1])
+    briefMessage = levels[levelIndex + 1]
+    fullMessage =  "Debug level %s." % briefMessage
+    orca_state.activeScript.presentMessage(fullMessage, briefMessage)
 
     return True
 
@@ -368,11 +381,33 @@ def exitLearnMode(self, inputEvent=None):
     # mode.
     #
     message = _("Exiting learn mode.")
-    speech.speak(message)
-    braille.displayMessage(message)
+    orca_state.activeScript.presentMessage(message)
     settings.learnModeEnabled = False
     return True
 
+def exitListShortcutsMode(self, inputEvent=None):
+    """Turns list shortcuts mode off.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    orca_state.listOfShortcuts = []
+    orca_state.typeOfShortcuts = ""
+    orca_state.ptrToShortcut = -1
+    settings.listShortcutsModeEnabled = False
+
+    # Translators: Orca has a "List Shortcuts Mode" that allows the user to
+    # list a group of keyboard shortcuts. Pressing 1 makes it possible for
+    # the user to navigate amongst a list of global ("default") commands.
+    # Pressing 2 allows the user to navigate amongst Orca commands specific
+    # to the application with focus. Escape exists this mode. This string
+    # is the prompt which will be presented to the user in both speech and
+    # braille upon exiting this mode.
+    #
+    message = _("Exiting list shortcuts mode.")
+    orca_state.activeScript.presentMessage(message)
+    return True
+
 ########################################################################
 #                                                                      #
 # METHODS FOR PRE-PROCESSING AND MASSAGING KEYBOARD EVENTS.            #
@@ -424,7 +459,9 @@ def isModifierKey(event_string):
 
     modifierKeys = [ 'Alt_L', 'Alt_R', 'Control_L', 'Control_R', \
                      'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R' ]
-    modifierKeys.extend(settings.orcaModifierKeys)
+
+    if not orca_state.bypassNextCommand:
+        modifierKeys.extend(settings.orcaModifierKeys)
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
@@ -442,8 +479,9 @@ def isLockingKey(event_string):
 
     lockingKeys = [ "Caps_Lock", "Num_Lock", "Scroll_Lock" ]
 
-    reply = event_string in lockingKeys \
-            and not event_string in settings.orcaModifierKeys
+    reply = event_string in lockingKeys
+    if not orca_state.bypassNextCommand:
+        reply = reply and not event_string in settings.orcaModifierKeys
     debug.println(debug.LEVEL_FINEST,
                   "orca.isLockingKey: returning: %s" % reply)
     return reply
@@ -499,7 +537,9 @@ def isNavigationKey(event_string):
 
     navigationKeys = [ "Left", "Right", "Up", "Down" ]
 
-    reply = (event_string in navigationKeys) or _orcaModifierPressed
+    reply = event_string in navigationKeys
+    if not reply and not orca_state.bypassNextCommand:
+        reply = _orcaModifierPressed
 
     debug.println(debug.LEVEL_FINEST,
                   "orca.isNavigationKey: returning: %s" % reply)
@@ -665,7 +705,7 @@ def keyEcho(event):
             eventType == KeyEventType.PRINTABLE \
             and not _orcaModifierPressed \
             and orca_state.activeScript \
-            and orca_state.activeScript.willEchoCharacter(event)
+            and orca_state.activeScript.utilities.willEchoCharacter(event)
 
         # One last check for echoing -- a PRINTABLE key may have squeaked
         # through due to the enableEchoByCharacter check above.  We only
@@ -784,6 +824,11 @@ def _processKeyboardEvent(event):
 
     keyboardEvent = KeyboardEvent(event)
 
+    # Log the KeyboardEvent translated from the pyatspi keyboardEvent.
+    #
+    string = keyboardEvent.toString()
+    debug.printInputEvent(debug.LEVEL_FINE, string)
+
     if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
         braille.killFlash()
 
@@ -872,7 +917,11 @@ def _processKeyboardEvent(event):
         if orca_state.capturingKeys:
             consumed = _processKeyCaptured(keyboardEvent)
         else:
-            consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
+            if settings.listShortcutsModeEnabled:
+                consumed = listShortcuts(keyboardEvent)
+            if (not consumed):
+                consumed = _keyBindings.consumeKeyboardEvent( \
+                  None, keyboardEvent)
             if (not consumed) and (_currentPresentationManager >= 0):
                 consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
                            processKeyboardEvent(keyboardEvent)
@@ -897,6 +946,16 @@ def _processKeyboardEvent(event):
 
                 consumed = True
 
+            global _restoreOrcaKeys
+            if not consumed \
+               and keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT:
+                if isOrcaModifier and orca_state.bypassNextCommand:
+                    _restoreXmodmap()
+                    _restoreOrcaKeys = True
+                elif _restoreOrcaKeys and not orca_state.bypassNextCommand:
+                    _createOrcaXmodmap()
+                    _restoreOrcaKeys = False
+
             if not consumed \
                and not isModifierKey(keyboardEvent.event_string) \
                and keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
@@ -958,15 +1017,78 @@ def toggleSilenceSpeech(script=None, inputEvent=None):
         # Translators: this is a spoken prompt letting the user know
         # that speech synthesis has been turned back on.
         #
-        speech.speak(_("Speech enabled."))
+        orca_state.activeScript.presentMessage(_("Speech enabled."))
     else:
         # Translators: this is a spoken prompt letting the user know
         # that speech synthesis has been temporarily turned off.
         #
-        speech.speak(_("Speech disabled."))
+        orca_state.activeScript.presentMessage(_("Speech disabled."))
         settings.silenceSpeech = True
     return True
 
+def _createOrcaXmodmap():
+    """Makes an Orca-specific Xmodmap so that the keys behave as we
+    need them to do. This is especially the case for the Orca modifier.
+    """
+
+    global _capsLockCleared
+
+    cmd = []
+    if "Caps_Lock" in settings.orcaModifierKeys:
+        cmd.append("clear Lock")
+        _capsLockCleared = True
+    elif _capsLockCleared:
+        cmd.append("add Lock = Caps_Lock")
+        _capsLockCleared = False
+
+    # Clear other keysyms so that we always treat the Orca modifier as
+    # the Orca modifier (e.g. remove KP_0 from KP_Insert).
+    #
+    for keyName in settings.orcaModifierKeys:
+        if keyName in ["Caps_Lock", "KP_Insert", "Insert"]:
+            cmd.append("keysym %s = %s" % (keyName, keyName))
+
+    os.system("echo '%s' | xmodmap - > /dev/null 2>&1" % "\n".join(cmd))
+
+def _storeXmodmap(keyList):
+    """Save the original xmodmap for the keys in keyList before we alter it.
+
+    Arguments:
+    - keyList: A list of named keys to look for.
+    """
+
+    global _originalXmodmap
+
+    items = "|".join(keyList)
+    cmd = "xmodmap -pke | grep -E '(%s)'" % items
+    filehandle = os.popen(cmd)
+    _originalXmodmap = filehandle.read()
+    filehandle.close()
+
+def _restoreXmodmap(keyList=[]):
+    """Restore the original xmodmap values for the keys in keyList.
+
+    Arguments:
+    - keyList: A list of named keys to look for. An empty list means
+      to restore the entire saved xmodmap.
+    """
+
+    toRestore = []
+    lines = _originalXmodmap.split("\n")
+    if not keyList:
+        toRestore = lines
+
+    for key in keyList:
+        line = filter(lambda k: " %s" % key in k, lines)
+        toRestore.extend(line)
+
+    global _capsLockCleared
+    if _capsLockCleared:
+        toRestore.append("add Lock = Caps_Lock")
+        _capsLockCleared = False
+
+    os.system("echo '%s' | xmodmap - > /dev/null 2>&1" % "\n".join(toRestore))
+
 def loadUserSettings(script=None, inputEvent=None):
     """Loads (and reloads) the user settings module, reinitializing
     things such as speech if necessary.
@@ -1030,7 +1152,8 @@ def loadUserSettings(script=None, inputEvent=None):
                 # preferences.  This is a spoken prompt to let the user
                 # know when the preferences has been reloaded.
                 #
-                speech.speak(_("Orca user settings reloaded."))
+                msg = _("Orca user settings reloaded.")
+                speech.speak(msg, settings.voices.get(settings.SYSTEM_VOICE))
             debug.println(debug.LEVEL_CONFIGURATION,
                           "Speech module has been initialized.")
         except:
@@ -1069,30 +1192,16 @@ def loadUserSettings(script=None, inputEvent=None):
     except NameError:
         pass
 
-    # We don't want the Caps_Lock modifier to act as a locking
-    # modifier if it used as the Orca modifier key.  In addition, if
-    # the KP_Insert key is used as the Orca modifier key, we want to
-    # make sure we clear any other keysyms that might be in use on
-    # that key since we won't be able to detect them as being the Orca
-    # modifier key.  For example, KP_Insert produces "KP_Insert" when
-    # pressed by itself, but Shift+KP_Insert produces "0".
+    global _orcaModifiers
+    custom = [k for k in settings.orcaModifierKeys if k not in _orcaModifiers]
+    _orcaModifiers += custom
+    # Handle the case where a change was made in the Orca Preferences dialog.
     #
-    # The original values are saved/reset in the orca shell script.
-    #
-    # [[[TODO: WDW - we probably should just to a 'xmodmap -e "%s = %s"'
-    # for all of the orcaModifierKeys, but saving/restoring the values
-    # becomes a little more difficult.  If we could assume a writeable
-    # filesystem (we cannot), we could do a 'xmodmap -pke > /tmp/foo'
-    # to save the keymap and a 'xmodmap /tmp/foo' to restore it.
-    # For now, we'll just look at the Orca modifier keys we support
-    # (Caps Lock, KP_Insert, and Insert).]]]
-    #
-    for keyName in settings.orcaModifierKeys:
-        if keyName == "Caps_Lock":
-            os.system('xmodmap -e "clear Lock"')
-        if keyName in ["Caps_Lock", "KP_Insert", "Insert"]:
-            command = 'xmodmap -e "keysym %s = %s"' % (keyName, keyName)
-            os.system(command)
+    if _originalXmodmap:
+        _restoreXmodmap(_orcaModifiers)
+
+    _storeXmodmap(_orcaModifiers)
+    _createOrcaXmodmap()
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].activate()
@@ -1194,6 +1303,221 @@ def helpForOrca(script=None, inputEvent=None):
                  gtk.get_current_event_time())
     return True
 
+def listShortcuts(event):
+    """When list shortcuts mode is enabled, this function provides a means
+    by which users can navigate through Orca bound Orca commands. Pressing
+    1 results in a list of the default shortcuts; pressing 2 results in a
+    list of shortcuts for the focused application, should one exist. List
+    navigation is accomplished through the Up and Down Arrow keys. Escape
+    exits the list. In this mode, other keys are disabled.
+
+    Arguments:
+    - event: an AT-SPI DeviceEvent
+
+    Returns True if the event is consumed (and False if not).
+    """
+
+    numShortcuts = len(orca_state.listOfShortcuts)
+    consumed = False
+    clickCount = 0
+    message = ""
+
+    # Translators: The following string instructs the user how to navigate
+    # amongst the list of commands presented in 'list shortcuts' mode as
+    # well as how to exit the list when finished.
+    #
+    navigation = \
+        _("Use Up and Down Arrow to navigate the list. Press Escape to exit.")
+
+    if event.type == pyatspi.KEY_PRESSED_EVENT:
+        clickCount = orca_state.activeScript.getClickCount()
+        if (event.event_string == "1"):
+            if not numShortcuts or orca_state.typeOfShortcuts != "default":
+                orca_state.listOfShortcuts = getListOfShortcuts("default")
+                orca_state.typeOfShortcuts = "default"
+                numShortcuts = len(orca_state.listOfShortcuts)
+            orca_state.ptrToShortcut = 0
+            # Translators: This message is presented when the user is in
+            # 'list of shortcuts mode'. In this messsage, we present the
+            # number of shortcuts found.
+            #
+            message = ngettext("%d Orca default shortcut found.",
+                               "%d Orca default shortcuts found.",
+                               numShortcuts) % numShortcuts
+            message = message + " " + navigation
+            orca_state.activeScript.presentMessage(message)
+            message = orca_state.listOfShortcuts[orca_state.ptrToShortcut][0]+ \
+              " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+            orca_state.activeScript.speakMessage(message)
+            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "2"):
+            if not numShortcuts or orca_state.typeOfShortcuts != "application":
+                orca_state.listOfShortcuts = getListOfShortcuts("application")
+                orca_state.typeOfShortcuts = "application"
+                numShortcuts = len(orca_state.listOfShortcuts)
+            if numShortcuts > 0: 
+                orca_state.ptrToShortcut = 0
+                # Translators: This message is presented when the user is in
+                # 'list of shortcuts mode'. In this message, we present the
+                # number of shortcuts found for the named application.
+                #
+                message = ngettext( \
+                    "%(count)d Orca shortcut for %(application)s found.",
+                    "%(count)d Orca shortcuts for %(application)s found.",
+                    numShortcuts) % \
+                    {"count" : numShortcuts,
+                     "application" : orca_state.activeScript.app.name}
+                message = message + " " + navigation
+                orca_state.activeScript.presentMessage(message)
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                orca_state.activeScript.speakMessage(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            else:
+                # Translators: This message is presented when the user is in
+                # 'list of shortcuts mode'. This is the message we present
+                # when the user requested a list of application-specific
+                # shortcuts, but none could be found for that application.
+                #
+                message = _("No Orca shortcuts for %s found.") % \
+                    (orca_state.activeScript.app.name)
+                orca_state.activeScript.speakMessage(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Up"):
+            if (numShortcuts > 0):
+                if orca_state.ptrToShortcut > 0: 
+                    orca_state.ptrToShortcut = orca_state.ptrToShortcut-1
+                else:
+                    orca_state.ptrToShortcut = numShortcuts-1 
+                    # Translators: when the user is attempting to locate a
+                    # particular object and the top of a page or list is
+                    # reached without that object being found, we "wrap" to
+                    # the bottom and continue looking upwards. We need to
+                    # inform the user when this is taking place.
+                    #
+                    orca_state.activeScript.\
+                        presentMessage(_("Wrapping to bottom."))
+
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                orca_state.activeScript.speakMessage(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Down"):
+            if (numShortcuts > 0): 
+                if orca_state.ptrToShortcut < numShortcuts-1: 
+                    orca_state.ptrToShortcut = orca_state.ptrToShortcut+1
+                else:
+                    orca_state.ptrToShortcut = 0 
+                    # Translators: when the user is attempting to locate a
+                    # particular object and the bottom of a page or list is
+                    # reached without that object being found, we "wrap" to the
+                    # top and continue looking downwards. We need to inform the
+                    # user when this is taking place.
+                    #
+                    orca_state.activeScript.\
+                        presentMessage(_("Wrapping to top."))
+                message = \
+                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
+                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
+                orca_state.activeScript.speakMessage(message)
+                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+        elif (event.event_string == "Escape"):
+            exitListShortcutsMode(event)
+            consumed = True 
+        else:
+            # Translators: Orca has a 'List Shortcuts' mode by which a user can
+            # navigate through a list of the bound commands in Orca. Pressing 1
+            # presents the commands/shortcuts available for all applications.
+            # These are the "default" commands/shortcuts. Pressing 2 presents
+            # commands/shortcuts Orca provides for the application with focus.
+            # The following message is presented to the user upon entering this
+            # mode.
+            #
+            message = _("Press 1 for Orca's default shortcuts. Press 2 for " \
+                        "Orca's shortcuts for the current application. " \
+                        "Press escape to exit.")
+            orca_state.activeScript.speakMessage(message)
+            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
+            consumed = True
+    elif (event.type == pyatspi.KEY_RELEASED_EVENT) and (event.event_string \
+      == "Escape"):
+        consumed = True
+    return consumed
+
+
+def getListOfShortcuts(typeOfShortcuts):
+    """This function returns a list of Orca default shortcuts if the argument
+    is 'default'. It returns a list of Orca shortcuts which are specific to
+    the focused application, if the argument is "application". Orca default
+    shortcuts are those found in the default script. Application-specific
+    shortcuts are those which are present in the active script, but not in
+    the default script. Only one shortcut per handler is listed. The list is
+    sorted on shortcuts.
+
+    Arguments:
+    - typeOfShortcuts: a string specifying the desired type of shortcuts.
+
+    Returns a list of shortcuts; depending on the value of argument.
+    """
+
+    import default
+
+    numShortcuts = len(orca_state.listOfShortcuts)
+    shortcuts = []
+    shortcut = ""
+    clickCount = ""
+    brlKeyName = ""     
+    brlHandler = None
+    defScript = default.Script(None)
+    defKeyBindings = defScript.getKeyBindings()
+    defBrlBindings = defScript.getBrailleBindings()
+    kbindings = keybindings.KeyBindings()
+    if typeOfShortcuts == "default":
+        for kb in defKeyBindings.keyBindings:
+            if kb.keysymstring:
+                if not kbindings.hasKeyBinding(kb,"description"):
+                    kbindings.add(kb)
+    elif typeOfShortcuts == "application":
+        for kb in orca_state.activeScript.keyBindings.keyBindings:
+            if kb.keysymstring:
+                if not (defKeyBindings.hasKeyBinding(kb,"description") or \
+                kbindings.hasKeyBinding(kb,"description")):
+                    kbindings.add(kb)
+
+    for kb in kbindings.keyBindings:
+        keysymString = kb.keysymstring.replace("KP_", _("keypad "))
+        clickCount = ""
+        if kb.click_count == 2:
+            # Translators: Orca keybindings support double
+            # and triple "clicks" or key presses, similar to
+            # using a mouse.
+            #
+            clickCount = _("double click")
+        elif kb.click_count == 3:
+            # Translators: Orca keybindings support double
+            # and triple "clicks" or key presses, similar to
+            # using a mouse.
+            #
+            clickCount = _("triple click")
+        shortcut = (kb.handler.description, keybindings.getModifierNames\
+        (kb.modifiers) + keysymString.upper() + " " + clickCount)
+        shortcuts.append(shortcut)
+        shortcuts = sorted(shortcuts, key=lambda shortcut: shortcut[1])
+
+    if typeOfShortcuts == "orcaDefault":
+        for bb in defBrlBindings.keys():
+            brlKeyName = braille.command_name[bb]
+            brlHandler = defBrlBindings[bb]
+            shortcut = (brlHandler.description, brlKeyName)
+            shortcuts.append(shortcut) 
+    return shortcuts
+
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
@@ -1230,6 +1554,24 @@ def showFindGUI(script=None, inputEvent=None):
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
+def showSplashGUI(script=None, inputEvent=None):
+    """Displays a splash screen.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    try:
+        module = __import__(settings.splashModule,
+                            globals(),
+                            locals(),
+                            [''])
+        module.showSplashUI()
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+
+    return True
+
+
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -1271,6 +1613,8 @@ def init(registry):
     registry.registerEventListener(_onMouseButton,
                                    "mouse:button")
 
+    showSplashGUI()
+
     loadUserSettings()
 
     masks = []
@@ -1357,8 +1701,7 @@ def shutdown(script=None, inputEvent=None):
 
     # Translators: this is what Orca speaks and brailles when it quits.
     #
-    speech.speak(_("Goodbye."))
-    braille.displayMessage(_("Goodbye."))
+    orca_state.activeScript.presentMessage(_("Goodbye."))
 
     # Deregister our event listeners
     #
@@ -1385,6 +1728,8 @@ def shutdown(script=None, inputEvent=None):
         signal.alarm(0)
 
     _initialized = False
+    _restoreXmodmap(_orcaModifiers)
+
     return True
 
 exitCount = 0
@@ -1447,7 +1792,7 @@ def usage():
     #
     print "-?, --help                   " + _("Show this help message")
 
-    print "-v, --version                %s" % platform.version
+    print "-v, --version                %s" % orca_platform.version
 
     # Translators: this is a testing option for the command line.  It prints
     # the names of the applications known to the accessibility infrastructure
@@ -1496,11 +1841,9 @@ def usage():
     print "-n, --no-setup               " +  \
           _("Skip set up of user preferences")
 
-    # Translators: by default, Orca expects to find its user preferences
-    # in a directory called .orca under the user's home directory. This
-    # is the description of the command line option
+    # Translators: this is the description of the command line option
     # '-u, --user-prefs-dir=dirname' that allows you to specify an alternate
-    # location for those user preferences.
+    # location for the user preferences.
     #
     print "-u, --user-prefs-dir=dirname " + \
           _("Use alternate directory for user preferences")
@@ -1539,6 +1882,18 @@ def usage():
     print "-q, --quit                   " + \
           _("Quits Orca (if shell script used)")
 
+    # Translators: this is the Orca command line option that will force 
+    # the termination of Orca.
+    # The user would run the Orca shell script again from a terminal window.
+    # If this command line option is specified, the script will quit any
+    # instances of Orca that are already running.
+    #
+    print "-f, --forcequit              " + \
+          _("Forces orca to be terminated immediately.")
+
+    print "-m, --migrate-config         " +\
+          "Move the user's preferences from ~/.orca to XDG-DATA-HOME/orca."
+
     # Translators: this is the Orca command line option to tell Orca to
     # replace any existing Orca process(es) that might be running.
     #
@@ -1622,6 +1977,7 @@ def main():
         # d is for disabling a feature
         # h is for help
         # u is for alternate user preferences location
+        # m is for migrate the user preferences location from ~/.orca
         # s is for setup
         # n is for no setup
         # t is for text setup
@@ -1630,7 +1986,7 @@ def main():
         #
         opts, args = getopt.getopt(
             arglist,
-            "?stnvld:e:u:",
+            "?stnvlmd:e:u:",
             ["help",
              "user-prefs-dir=",
              "enable=",
@@ -1643,6 +1999,7 @@ def main():
              "debug",
              "debug-file=",
              "version",
+             "migrate-config",
              "replace"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
@@ -1653,6 +2010,22 @@ def main():
                 except:
                     debug.printException(debug.LEVEL_FINEST)
 
+            if opt in ("-m", "--migrate-config"):
+                from xdg.BaseDirectory import xdg_data_home
+                userPrefsDir = os.path.join(xdg_data_home, "orca")
+                oldUserPrefsDir = os.path.join(os.environ["HOME"], ".orca")
+                if os.path.exists(oldUserPrefsDir):
+                    try:
+                        os.renames(oldUserPrefsDir, userPrefsDir)
+                    except:
+                       print "It seems like you are trying to migrate your " \
+                           + "preferences but the new location is not " \
+                           + "empty.\n" \
+                           + "Probably it was already migrated. \n" \
+                           + "Please, check it before you try again."
+                       die(2)
+                settings.userPrefsDir = userPrefsDir
+
             if opt in ("-e", "--enable"):
                 feature = val.strip()
 
@@ -1698,7 +2071,7 @@ def main():
                 usage()
                 die(0)
             if opt in ("-v", "--version"):
-                print "Orca %s" % platform.version
+                print "Orca %s" % orca_platform.version
                 die(0)
             if opt == "--debug":
                 _debugSwitch = True
@@ -1748,7 +2121,7 @@ def main():
 
     try:
         message = _("Welcome to Orca.")
-        speech.speak(message)
+        speech.speak(message, settings.voices.get(settings.SYSTEM_VOICE))
         braille.displayMessage(message)
     except:
         debug.printException(debug.LEVEL_SEVERE)
@@ -1763,6 +2136,13 @@ def main():
         showPreferencesGUI()
     elif (not _userSettings) and (not bypassSetup):
         if desktopRunning:
+            if not os.path.exists(userprefs):
+                # Hack to work around b.g.o. 601657.
+                #
+                try:
+                    os.mkdir(userprefs)
+                except:
+                    debug.printException(debug.LEVEL_FINEST)
             showPreferencesGUI()
         else:
             _showPreferencesConsole()
